@@ -1,97 +1,160 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
+
+from __future__ import annotations
 
 import csv
 import operator
 import platform
 import sys
 from collections import Counter
-from io import StringIO, TextIOWrapper
-from typing import TYPE_CHECKING, Dict, List, Optional, Tuple
+from io import StringIO
+from pathlib import Path
+from typing import Counter as CounterType
+from typing import TextIO, Tuple
 
 import pytest
 from _pytest.config import Config
 
 from pylint import checkers
+from pylint.config.config_initialization import _config_initialization
+from pylint.constants import IS_PYPY
 from pylint.lint import PyLinter
+from pylint.message.message import Message
 from pylint.testutils.constants import _EXPECTED_RE, _OPERATORS, UPDATE_OPTION
-from pylint.testutils.functional_test_file import (
+from pylint.testutils.functional.test_file import (  # need to import from functional.test_file to avoid cyclic import
     FunctionalTestFile,
     NoFileError,
     parse_python_version,
 )
 from pylint.testutils.output_line import OutputLine
 from pylint.testutils.reporter_for_tests import FunctionalTestReporter
-from pylint.utils import utils
 
-if TYPE_CHECKING:
-    from typing import Counter as CounterType  # typing.Counter added in Python 3.6.1
+MessageCounter = CounterType[Tuple[int, str]]
+
+PYLINTRC = Path(__file__).parent / "testing_pylintrc"
 
 
 class LintModuleTest:
     maxDiff = None
 
-    def __init__(self, test_file: FunctionalTestFile, config: Optional[Config] = None):
+    def __init__(
+        self, test_file: FunctionalTestFile, config: Config | None = None
+    ) -> None:
         _test_reporter = FunctionalTestReporter()
         self._linter = PyLinter()
-        self._linter.set_reporter(_test_reporter)
         self._linter.config.persistent = 0
         checkers.initialize(self._linter)
-        self._linter.disable("suppressed-message")
-        self._linter.disable("locally-disabled")
-        self._linter.disable("useless-suppression")
+
+        # See if test has its own .rc file, if so we use that one
+        rc_file: Path | str = PYLINTRC
         try:
-            self._linter.read_config_file(test_file.option_file)
-            if self._linter.cfgfile_parser.has_option("MASTER", "load-plugins"):
-                plugins = utils._splitstrip(
-                    self._linter.cfgfile_parser.get("MASTER", "load-plugins")
-                )
-                self._linter.load_plugin_modules(plugins)
-            self._linter.load_config_file()
+            rc_file = test_file.option_file
+            self._linter.disable("suppressed-message")
+            self._linter.disable("locally-disabled")
+            self._linter.disable("useless-suppression")
         except NoFileError:
             pass
+
         self._test_file = test_file
+        try:
+            args = [test_file.source]
+        except NoFileError:
+            # If we're still raising NoFileError the actual source file doesn't exist
+            args = [""]
+        if config and config.getoption("minimal_messages_config"):
+            with self._open_source_file() as f:
+                messages_to_enable = {msg[1] for msg in self.get_expected_messages(f)}
+                # Always enable fatal errors
+                messages_to_enable.add("astroid-error")
+                messages_to_enable.add("fatal")
+                messages_to_enable.add("syntax-error")
+            args.extend(["--disable=all", f"--enable={','.join(messages_to_enable)}"])
+
+        # Add testoptions
+        self._linter._arg_parser.add_argument(
+            "--min_pyver", type=parse_python_version, default=(2, 5)
+        )
+        self._linter._arg_parser.add_argument(
+            "--max_pyver", type=parse_python_version, default=(4, 0)
+        )
+        self._linter._arg_parser.add_argument(
+            "--min_pyver_end_position", type=parse_python_version, default=(3, 8)
+        )
+        self._linter._arg_parser.add_argument(
+            "--requires", type=lambda s: [i.strip() for i in s.split(",")], default=[]
+        )
+        self._linter._arg_parser.add_argument(
+            "--except_implementations",
+            type=lambda s: [i.strip() for i in s.split(",")],
+            default=[],
+        )
+        self._linter._arg_parser.add_argument(
+            "--exclude_platforms",
+            type=lambda s: [i.strip() for i in s.split(",")],
+            default=[],
+        )
+        self._linter._arg_parser.add_argument(
+            "--exclude_from_minimal_messages_config", default=False
+        )
+
+        _config_initialization(
+            self._linter, args_list=args, config_file=rc_file, reporter=_test_reporter
+        )
+
+        self._check_end_position = (
+            sys.version_info >= self._linter.config.min_pyver_end_position
+        )
+        # TODO: PY3.9: PyPy supports end_lineno from 3.9 and above
+        if self._check_end_position and IS_PYPY:
+            self._check_end_position = sys.version_info >= (3, 9)  # pragma: no cover
+
         self._config = config
 
-    def setUp(self):
+    def setUp(self) -> None:
         if self._should_be_skipped_due_to_version():
             pytest.skip(
                 f"Test cannot run with Python {sys.version.split(' ', maxsplit=1)[0]}."
             )
         missing = []
-        for requirement in self._test_file.options["requires"]:
+        for requirement in self._linter.config.requires:
             try:
                 __import__(requirement)
             except ImportError:
                 missing.append(requirement)
         if missing:
             pytest.skip(f"Requires {','.join(missing)} to be present.")
-        except_implementations = self._test_file.options["except_implementations"]
+        except_implementations = self._linter.config.except_implementations
         if except_implementations:
-            implementations = [i.strip() for i in except_implementations.split(",")]
-            if platform.python_implementation() in implementations:
+            if platform.python_implementation() in except_implementations:
                 msg = "Test cannot run with Python implementation %r"
                 pytest.skip(msg % platform.python_implementation())
-        excluded_platforms = self._test_file.options["exclude_platforms"]
+        excluded_platforms = self._linter.config.exclude_platforms
         if excluded_platforms:
-            platforms = [p.strip() for p in excluded_platforms.split(",")]
-            if sys.platform.lower() in platforms:
+            if sys.platform.lower() in excluded_platforms:
                 pytest.skip(f"Test cannot run on platform {sys.platform!r}")
+        if (
+            self._config
+            and self._config.getoption("minimal_messages_config")
+            and self._linter.config.exclude_from_minimal_messages_config
+        ):
+            pytest.skip("Test excluded from --minimal-messages-config")
 
-    def runTest(self):
+    def runTest(self) -> None:
         self._runTest()
 
-    def _should_be_skipped_due_to_version(self):
+    def _should_be_skipped_due_to_version(self) -> bool:
         return (
-            sys.version_info < self._test_file.options["min_pyver"]
-            or sys.version_info > self._test_file.options["max_pyver"]
+            sys.version_info < self._linter.config.min_pyver
+            or sys.version_info > self._linter.config.max_pyver
         )
 
-    def __str__(self):
+    def __str__(self) -> str:
         return f"{self._test_file.base} ({self.__class__.__module__}.{self.__class__.__name__})"
 
     @staticmethod
-    def get_expected_messages(stream: TextIOWrapper) -> "CounterType[Tuple[int, str]]":
+    def get_expected_messages(stream: TextIO) -> MessageCounter:
         """Parses a file and get expected messages.
 
         :param stream: File-like input stream.
@@ -99,7 +162,7 @@ class LintModuleTest:
         :returns: A dict mapping line,msg-symbol tuples to the count on this line.
         :rtype: dict
         """
-        messages: "CounterType[Tuple[int, str]]" = Counter()
+        messages: MessageCounter = Counter()
         for i, line in enumerate(stream):
             match = _EXPECTED_RE.search(line)
             if match is None:
@@ -125,11 +188,13 @@ class LintModuleTest:
 
     @staticmethod
     def multiset_difference(
-        expected_entries: "CounterType", actual_entries: "CounterType"
-    ) -> Tuple["CounterType", Dict[str, int]]:
+        expected_entries: MessageCounter,
+        actual_entries: MessageCounter,
+    ) -> tuple[MessageCounter, dict[tuple[int, str], int]]:
         """Takes two multisets and compares them.
 
-        A multiset is a dict with the cardinality of the key as the value."""
+        A multiset is a dict with the cardinality of the key as the value.
+        """
         missing = expected_entries.copy()
         missing.subtract(actual_entries)
         unexpected = {}
@@ -140,46 +205,47 @@ class LintModuleTest:
                     unexpected[key] = -value
         return missing, unexpected
 
-    # pylint: disable=consider-using-with
-    def _open_expected_file(self):
+    def _open_expected_file(self) -> TextIO:
         try:
             return open(self._test_file.expected_output, encoding="utf-8")
         except FileNotFoundError:
             return StringIO("")
 
-    # pylint: disable=consider-using-with
-    def _open_source_file(self):
+    def _open_source_file(self) -> TextIO:
         if self._test_file.base == "invalid_encoded_data":
             return open(self._test_file.source, encoding="utf-8")
         if "latin1" in self._test_file.base:
             return open(self._test_file.source, encoding="latin1")
         return open(self._test_file.source, encoding="utf8")
 
-    def _get_expected(self):
+    def _get_expected(self) -> tuple[MessageCounter, list[OutputLine]]:
         with self._open_source_file() as f:
             expected_msgs = self.get_expected_messages(f)
         if not expected_msgs:
             expected_msgs = Counter()
         with self._open_expected_file() as f:
             expected_output_lines = [
-                OutputLine.from_csv(row) for row in csv.reader(f, "test")
+                OutputLine.from_csv(row, self._check_end_position)
+                for row in csv.reader(f, "test")
             ]
         return expected_msgs, expected_output_lines
 
-    def _get_actual(self):
-        messages = self._linter.reporter.messages
+    def _get_actual(self) -> tuple[MessageCounter, list[OutputLine]]:
+        messages: list[Message] = self._linter.reporter.messages
         messages.sort(key=lambda m: (m.line, m.symbol, m.msg))
-        received_msgs = Counter()
+        received_msgs: MessageCounter = Counter()
         received_output_lines = []
         for msg in messages:
             assert (
                 msg.symbol != "fatal"
             ), f"Pylint analysis failed because of '{msg.msg}'"
             received_msgs[msg.line, msg.symbol] += 1
-            received_output_lines.append(OutputLine.from_msg(msg))
+            received_output_lines.append(
+                OutputLine.from_msg(msg, self._check_end_position)
+            )
         return received_msgs, received_output_lines
 
-    def _runTest(self):
+    def _runTest(self) -> None:
         __tracebackhide__ = True  # pylint: disable=unused-variable
         modules_to_check = [self._test_file.source]
         self._linter.check(modules_to_check)
@@ -193,28 +259,32 @@ class LintModuleTest:
         self._check_output_text(expected_messages, expected_output, actual_output)
 
     def error_msg_for_unequal_messages(
-        self, actual_messages, expected_messages, actual_output: List[OutputLine]
-    ):
+        self,
+        actual_messages: MessageCounter,
+        expected_messages: MessageCounter,
+        actual_output: list[OutputLine],
+    ) -> str:
         msg = [f'Wrong results for file "{self._test_file.base}":']
         missing, unexpected = self.multiset_difference(
             expected_messages, actual_messages
         )
         if missing:
             msg.append("\nExpected in testdata:")
-            msg.extend(
-                " %3d: %s" % msg  # pylint: disable=consider-using-f-string
-                for msg in sorted(missing)
-            )
+            msg.extend(f" {msg[0]:3}: {msg[1]}" for msg in sorted(missing))
         if unexpected:
             msg.append("\nUnexpected in testdata:")
-            msg.extend(" %3d: %s" % msg for msg in sorted(unexpected))  # type: ignore #pylint: disable=consider-using-f-string
+            msg.extend(f" {msg[0]:3}: {msg[1]}" for msg in sorted(unexpected))
         error_msg = "\n".join(msg)
         if self._config and self._config.getoption("verbose") > 0:
             error_msg += "\n\nActual pylint output for this file:\n"
             error_msg += "\n".join(str(o) for o in actual_output)
         return error_msg
 
-    def error_msg_for_unequal_output(self, expected_lines, received_lines) -> str:
+    def error_msg_for_unequal_output(
+        self,
+        expected_lines: list[OutputLine],
+        received_lines: list[OutputLine],
+    ) -> str:
         missing = set(expected_lines) - set(received_lines)
         unexpected = set(received_lines) - set(expected_lines)
         error_msg = (
@@ -234,8 +304,15 @@ class LintModuleTest:
                 error_msg += f"{line}\n"
         return error_msg
 
-    def _check_output_text(self, _, expected_output, actual_output):
-        """This is a function because we want to be able to update the text in LintModuleOutputUpdate"""
+    def _check_output_text(
+        self,
+        _: MessageCounter,
+        expected_output: list[OutputLine],
+        actual_output: list[OutputLine],
+    ) -> None:
+        """This is a function because we want to be able to update the text in
+        LintModuleOutputUpdate.
+        """
         assert expected_output == actual_output, self.error_msg_for_unequal_output(
             expected_output, actual_output
         )
